@@ -19,6 +19,8 @@
 //! The simplest way to construct a [`NonEmpty`] is via the [`nonempty`] macro:
 //!
 //! ```
+//! # extern crate alloc;
+//! # use alloc::vec::Vec;
 //! use nonempty::{NonEmpty, nonempty};
 //!
 //! let l: NonEmpty<u32> = nonempty![1, 2, 3];
@@ -28,6 +30,8 @@
 //! Unlike the familiar `vec!` macro, `nonempty!` requires at least one element:
 //!
 //! ```
+//! # extern crate alloc;
+//! # use alloc::vec::Vec;
 //! use nonempty::nonempty;
 //!
 //! let l = nonempty![1];
@@ -40,6 +44,8 @@
 //! [`NonEmpty::new`] or its constructor:
 //!
 //! ```
+//! # extern crate alloc;
+//! # use alloc::vec::Vec;
 //! use nonempty::NonEmpty;
 //!
 //! let mut l = NonEmpty { head: 42, tail: vec![36, 58] };
@@ -65,20 +71,33 @@
 //! # Caveats
 //!
 //! Since `NonEmpty` must have a least one element, it is not possible to
-//! implement the `FromInterator` trait for it. We can't know, in general, if
+//! implement the `FromIterator` trait for it. We can't know, in general, if
 //! any given `Iterator` actually contains something.
-//!
+
+#![no_std]
+
 //! # Features
 //!
 //! * `serialize`: `serde` support.
+//! * `arbitrary`: `arbitrary` support.
+#[cfg(feature = "arbitrary")]
+use arbitrary::Arbitrary;
 #[cfg(feature = "serialize")]
 use serde::{
     ser::{SerializeSeq, Serializer},
     Deserialize, Serialize,
 };
-use std::cmp::Ordering;
-use std::mem;
-use std::{iter, vec};
+
+use core::iter;
+use core::mem;
+use core::{cmp::Ordering, num::NonZeroUsize};
+
+#[cfg(feature = "std")]
+extern crate std;
+
+#[cfg_attr(test, macro_use)]
+extern crate alloc;
+use alloc::vec::{self, Vec};
 
 pub mod nonzero;
 
@@ -86,6 +105,8 @@ pub mod nonzero;
 /// for constructing [`NonEmpty`] values.
 ///
 /// ```
+/// # extern crate alloc;
+/// # use alloc::vec::Vec;
 /// use nonempty::{NonEmpty, nonempty};
 ///
 /// let v = nonempty![1, 2, 3];
@@ -94,25 +115,30 @@ pub mod nonzero;
 /// let v = nonempty![1];
 /// assert_eq!(v, NonEmpty { head: 1, tail: Vec::new() });
 ///
+/// // Accepts trailing commas
+/// let v = nonempty![1,];
+/// assert_eq!(v, NonEmpty { head: 1, tail: Vec::new() });
+///
 /// // Doesn't compile!
 /// // let v = nonempty![];
 /// ```
 #[macro_export]
 macro_rules! nonempty {
-    ($h:expr, $( $x:expr ),*) => {{
+    ($h:expr, $( $x:expr ),* $(,)?) => {{
         let tail = vec![$($x),*];
         $crate::NonEmpty { head: $h, tail }
     }};
     ($h:expr) => {
         $crate::NonEmpty {
             head: $h,
-            tail: Vec::new(),
+            tail: alloc::vec::Vec::new(),
         }
     };
 }
 
 /// Non-empty vector.
 #[cfg_attr(feature = "serialize", derive(Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(Arbitrary))]
 #[cfg_attr(feature = "serialize", serde(try_from = "Vec<T>"))]
 #[derive(Clone, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub struct NonEmpty<T> {
@@ -159,7 +185,7 @@ impl<'a, T> Iterator for Iter<'a, T> {
     }
 }
 
-impl<'a, T> DoubleEndedIterator for Iter<'a, T> {
+impl<T> DoubleEndedIterator for Iter<'_, T> {
     fn next_back(&mut self) -> Option<Self::Item> {
         if let Some((last, rest)) = self.tail.split_last() {
             self.tail = rest;
@@ -172,13 +198,13 @@ impl<'a, T> DoubleEndedIterator for Iter<'a, T> {
     }
 }
 
-impl<'a, T> ExactSizeIterator for Iter<'a, T> {
+impl<T> ExactSizeIterator for Iter<'_, T> {
     fn len(&self) -> usize {
         self.tail.len() + self.head.map_or(0, |_| 1)
     }
 }
 
-impl<'a, T> core::iter::FusedIterator for Iter<'a, T> {}
+impl<T> core::iter::FusedIterator for Iter<'_, T> {}
 
 impl<T> NonEmpty<T> {
     /// Alias for [`NonEmpty::singleton`].
@@ -186,6 +212,14 @@ impl<T> NonEmpty<T> {
         Self::singleton(e)
     }
 
+    /// Converts from `&NonEmpty<T>` to `NonEmpty<&T>`.
+    pub fn as_ref(&self) -> NonEmpty<&T> {
+        NonEmpty {
+            head: &self.head,
+            tail: self.tail.iter().collect(),
+        }
+    }
+
     /// Attempt to convert an iterator into a `NonEmpty` vector.
     /// Returns `None` if the iterator was empty.
     pub fn collect<I>(iter: I) -> Option<NonEmpty<T>>
@@ -300,6 +334,11 @@ impl<T> NonEmpty<T> {
         self.tail.len() + 1
     }
 
+    /// Gets the length of the list as a NonZeroUsize.
+    pub fn len_nonzero(&self) -> NonZeroUsize {
+        unsafe { NonZeroUsize::new_unchecked(self.tail.len().saturating_add(1)) }
+    }
+
     /// Get the capacity of the list.
     pub fn capacity(&self) -> usize {
         self.tail.capacity() + 1
@@ -400,7 +439,7 @@ impl<T> NonEmpty<T> {
     /// assert_eq!(l_iter.next(), Some(&580));
     /// assert_eq!(l_iter.next(), None);
     /// ```
-    pub fn iter_mut(&mut self) -> impl Iterator<Item = &mut T> + '_ {
+    pub fn iter_mut(&mut self) -> impl DoubleEndedIterator<Item = &mut T> + '_ {
         iter::once(&mut self.head).chain(self.tail.iter_mut())
     }
 
@@ -459,7 +498,7 @@ impl<T> NonEmpty<T> {
     }
 
     /// Deconstruct a `NonEmpty` into its head and tail.
-    /// This operation never fails since we are guranteed
+    /// This operation never fails since we are guaranteed
     /// to have a head element.
     ///
     /// # Example Use
@@ -484,7 +523,7 @@ impl<T> NonEmpty<T> {
     /// Deconstruct a `NonEmpty` into its first, last, and
     /// middle elements, in that order.
     ///
-    /// If there is only one element then first == last.
+    /// If there is only one element then last is `None`.
     ///
     /// # Example Use
     ///
@@ -493,19 +532,20 @@ impl<T> NonEmpty<T> {
     ///
     /// let mut non_empty = NonEmpty::from((1, vec![2, 3, 4, 5]));
     ///
-    /// // Guaranteed to have the last element and the elements
-    /// // preceding it.
-    /// assert_eq!(non_empty.split(), (&1, &[2, 3, 4][..], &5));
+    /// // When there are two or more elements, the last element is represented
+    /// // as a `Some`. Elements preceding it, except for the first, are returned
+    /// // in the middle.
+    /// assert_eq!(non_empty.split(), (&1, &[2, 3, 4][..], Some(&5)));
     ///
     /// let non_empty = NonEmpty::new(1);
     ///
-    /// // Guaranteed to have the last element.
-    /// assert_eq!(non_empty.split(), (&1, &[][..], &1));
+    /// // The last element is `None` when there's only one element.
+    /// assert_eq!(non_empty.split(), (&1, &[][..], None));
     /// ```
-    pub fn split(&self) -> (&T, &[T], &T) {
+    pub fn split(&self) -> (&T, &[T], Option<&T>) {
         match self.tail.split_last() {
-            None => (&self.head, &[], &self.head),
-            Some((last, middle)) => (&self.head, middle, last),
+            None => (&self.head, &[], None),
+            Some((last, middle)) => (&self.head, middle, Some(last)),
         }
     }
 
@@ -675,7 +715,7 @@ impl<T> NonEmpty<T> {
     /// # Examples
     ///
     /// Looks up a series of four elements. The first is found, with a uniquely determined
-    /// position; the second and third are not found; the fourth could match any position in [1,4].
+    /// position; the second and third are not found; the fourth could match any position in `[1,4]`.
     ///
     /// ```
     /// use nonempty::NonEmpty;
@@ -806,9 +846,9 @@ impl<T> NonEmpty<T> {
     /// let non_empty = NonEmpty::from(((2, 1), vec![(2, -34), (4, 42), (0, 76), (1, 4), (3, 5)]));
     /// assert_eq!(non_empty.maximum_by(|(k, _), (l, _)| k.cmp(l)), &(4, 42));
     /// ```
-    pub fn maximum_by<F>(&self, compare: F) -> &T
+    pub fn maximum_by<F>(&self, mut compare: F) -> &T
     where
-        F: Fn(&T, &T) -> Ordering,
+        F: FnMut(&T, &T) -> Ordering,
     {
         let mut max = &self.head;
         for i in self.tail.iter() {
@@ -834,9 +874,9 @@ impl<T> NonEmpty<T> {
     /// let non_empty = NonEmpty::from(((2, 1), vec![(2, -34), (4, 42), (0, 76), (1, 4), (3, 5)]));
     /// assert_eq!(non_empty.minimum_by(|(k, _), (l, _)| k.cmp(l)), &(0, 76));
     /// ```
-    pub fn minimum_by<F>(&self, compare: F) -> &T
+    pub fn minimum_by<F>(&self, mut compare: F) -> &T
     where
-        F: Fn(&T, &T) -> Ordering,
+        F: FnMut(&T, &T) -> Ordering,
     {
         self.maximum_by(|a, b| compare(a, b).reverse())
     }
@@ -851,17 +891,18 @@ impl<T> NonEmpty<T> {
     /// use nonempty::NonEmpty;
     ///
     /// let non_empty = NonEmpty::new((0, 42));
-    /// assert_eq!(non_empty.maximum_by_key(|(k, _)| k), &(0, 42));
+    /// assert_eq!(non_empty.maximum_by_key(|(k, _)| *k), &(0, 42));
     ///
     /// let non_empty = NonEmpty::from(((2, 1), vec![(2, -34), (4, 42), (0, 76), (1, 4), (3, 5)]));
-    /// assert_eq!(non_empty.maximum_by_key(|(k, _)| k), &(4, 42));
+    /// assert_eq!(non_empty.maximum_by_key(|(k, _)| *k), &(4, 42));
+    /// assert_eq!(non_empty.maximum_by_key(|(k, _)| -k), &(0, 76));
     /// ```
-    pub fn maximum_by_key<U, F>(&self, f: F) -> &T
+    pub fn maximum_by_key<U, F>(&self, mut f: F) -> &T
     where
         U: Ord,
-        F: Fn(&T) -> &U,
+        F: FnMut(&T) -> U,
     {
-        self.maximum_by(|i, j| f(i).cmp(f(j)))
+        self.maximum_by(|i, j| f(i).cmp(&f(j)))
     }
 
     /// Returns the element that gives the minimum value with respect to the specified function.
@@ -874,17 +915,52 @@ impl<T> NonEmpty<T> {
     /// use nonempty::NonEmpty;
     ///
     /// let non_empty = NonEmpty::new((0, 42));
-    /// assert_eq!(non_empty.minimum_by_key(|(k, _)| k), &(0, 42));
+    /// assert_eq!(non_empty.minimum_by_key(|(k, _)| *k), &(0, 42));
     ///
     /// let non_empty = NonEmpty::from(((2, 1), vec![(2, -34), (4, 42), (0, 76), (1, 4), (3, 5)]));
-    /// assert_eq!(non_empty.minimum_by_key(|(k, _)| k), &(0, 76));
+    /// assert_eq!(non_empty.minimum_by_key(|(k, _)| *k), &(0, 76));
+    /// assert_eq!(non_empty.minimum_by_key(|(k, _)| -k), &(4, 42));
     /// ```
-    pub fn minimum_by_key<U, F>(&self, f: F) -> &T
+    pub fn minimum_by_key<U, F>(&self, mut f: F) -> &T
     where
         U: Ord,
-        F: Fn(&T) -> &U,
+        F: FnMut(&T) -> U,
     {
-        self.minimum_by(|i, j| f(i).cmp(f(j)))
+        self.minimum_by(|i, j| f(i).cmp(&f(j)))
+    }
+
+    /// Sorts the nonempty.
+    ///
+    /// The implementation uses [`slice::sort`](slice::sort) for the tail and then checks where the
+    /// head belongs. If the head is already the smallest element, this should be as fast as sorting a
+    /// slice. However, if the head needs to be inserted, then it incurs extra cost for removing
+    /// the new head from the tail and adding the old head at the correct index.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use nonempty::nonempty;
+    ///
+    /// let mut non_empty = nonempty![-5, 4, 1, -3, 2];
+    ///
+    /// non_empty.sort();
+    /// assert!(non_empty == nonempty![-5, -3, 1, 2, 4]);
+    /// ```
+    pub fn sort(&mut self)
+    where
+        T: Ord,
+    {
+        self.tail.sort();
+        let index = match self.tail.binary_search(&self.head) {
+            Ok(index) => index,
+            Err(index) => index,
+        };
+
+        if index != 0 {
+            let new_head = self.tail.remove(0);
+            let head = mem::replace(&mut self.head, new_head);
+            self.tail.insert(index - 1, head);
+        }
     }
 }
 
@@ -927,14 +1003,14 @@ impl<T> IntoIterator for NonEmpty<T> {
 
 impl<'a, T> IntoIterator for &'a NonEmpty<T> {
     type Item = &'a T;
-    type IntoIter = iter::Chain<iter::Once<&'a T>, std::slice::Iter<'a, T>>;
+    type IntoIter = iter::Chain<iter::Once<&'a T>, core::slice::Iter<'a, T>>;
 
     fn into_iter(self) -> Self::IntoIter {
         iter::once(&self.head).chain(self.tail.iter())
     }
 }
 
-impl<T> std::ops::Index<usize> for NonEmpty<T> {
+impl<T> core::ops::Index<usize> for NonEmpty<T> {
     type Output = T;
 
     /// ```
@@ -955,7 +1031,7 @@ impl<T> std::ops::Index<usize> for NonEmpty<T> {
     }
 }
 
-impl<T> std::ops::IndexMut<usize> for NonEmpty<T> {
+impl<T> core::ops::IndexMut<usize> for NonEmpty<T> {
     fn index_mut(&mut self, index: usize) -> &mut T {
         if index > 0 {
             &mut self.tail[index - 1]
@@ -973,7 +1049,9 @@ impl<A> Extend<A> for NonEmpty<A> {
 
 #[cfg(feature = "serialize")]
 pub mod serialize {
-    use std::{convert::TryFrom, fmt};
+    use core::{convert::TryFrom, fmt};
+
+    use alloc::vec::Vec;
 
     use super::NonEmpty;
 
@@ -1003,6 +1081,8 @@ pub mod serialize {
 
 #[cfg(test)]
 mod tests {
+    use alloc::{string::String, vec::Vec};
+
     use crate::NonEmpty;
 
     #[test]
@@ -1034,14 +1114,15 @@ mod tests {
 
     #[test]
     fn test_iter_both_directions() {
-        let nonempty = NonEmpty::from((0, vec![1, 2, 3]));
-        assert_eq!(
-            nonempty.iter().cloned().collect::<Vec<_>>(),
-            vec![0, 1, 2, 3]
-        );
+        let mut nonempty = NonEmpty::from((0, vec![1, 2, 3]));
+        assert_eq!(nonempty.iter().cloned().collect::<Vec<_>>(), [0, 1, 2, 3]);
         assert_eq!(
             nonempty.iter().rev().cloned().collect::<Vec<_>>(),
-            vec![3, 2, 1, 0]
+            [3, 2, 1, 0]
+        );
+        assert_eq!(
+            nonempty.iter_mut().rev().collect::<Vec<_>>(),
+            [&mut 3, &mut 2, &mut 1, &mut 0]
         );
     }
 
@@ -1070,7 +1151,7 @@ mod tests {
 
     #[test]
     fn test_to_nonempty() {
-        use std::iter::{empty, once};
+        use core::iter::{empty, once};
 
         assert_eq!(NonEmpty::<()>::collect(empty()), None);
         assert_eq!(NonEmpty::<()>::collect(once(())), Some(NonEmpty::new(())));
@@ -1092,16 +1173,60 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_nontrivial_minimum_by_key() {
+        #[derive(Debug, Clone, Copy, PartialEq, Eq)]
+        struct Position {
+            x: i32,
+            y: i32,
+        }
+        impl Position {
+            pub fn distance_squared(&self, other: Position) -> u32 {
+                let dx = self.x - other.x;
+                let dy = self.y - other.y;
+                (dx * dx + dy * dy) as u32
+            }
+        }
+        let positions = nonempty![
+            Position { x: 1, y: 1 },
+            Position { x: 0, y: 0 },
+            Position { x: 3, y: 4 }
+        ];
+        let target = Position { x: 1, y: 2 };
+        let closest = positions.minimum_by_key(|position| position.distance_squared(target));
+        assert_eq!(closest, &Position { x: 1, y: 1 });
+    }
+
+    #[test]
+    fn test_sort() {
+        let mut numbers = nonempty![1];
+        numbers.sort();
+        assert_eq!(numbers, nonempty![1]);
+
+        let mut numbers = nonempty![2, 1, 3];
+        numbers.sort();
+        assert_eq!(numbers, nonempty![1, 2, 3]);
+
+        let mut numbers = nonempty![1, 3, 2];
+        numbers.sort();
+        assert_eq!(numbers, nonempty![1, 2, 3]);
+
+        let mut numbers = nonempty![3, 2, 1];
+        numbers.sort();
+        assert_eq!(numbers, nonempty![1, 2, 3]);
+    }
+
     #[cfg(feature = "serialize")]
     mod serialize {
         use crate::NonEmpty;
+        use alloc::boxed::Box;
         use serde::{Deserialize, Serialize};
 
         #[derive(Debug, Deserialize, Eq, PartialEq, Serialize)]
         pub struct SimpleSerializable(pub i32);
 
         #[test]
-        fn test_simple_round_trip() -> Result<(), Box<dyn std::error::Error>> {
+        fn test_simple_round_trip() -> Result<(), Box<dyn core::error::Error>> {
             // Given
             let mut non_empty = NonEmpty::new(SimpleSerializable(42));
             non_empty.push(SimpleSerializable(777));
@@ -1118,7 +1243,7 @@ mod tests {
         }
 
         #[test]
-        fn test_serialization() -> Result<(), Box<dyn std::error::Error>> {
+        fn test_serialization() -> Result<(), Box<dyn core::error::Error>> {
             let ne = nonempty![1, 2, 3, 4, 5];
             let ve = vec![1, 2, 3, 4, 5];
 
@@ -1127,4 +1252,51 @@ mod tests {
             Ok(())
         }
     }
+
+    #[cfg(feature = "arbitrary")]
+    mod arbitrary {
+        use crate::NonEmpty;
+        use arbitrary::{Arbitrary, Unstructured};
+
+        #[test]
+        fn test_arbitrary_empty_tail() -> arbitrary::Result<()> {
+            let mut u = Unstructured::new(&[1, 2, 3, 4]);
+            let ne = NonEmpty::<i32>::arbitrary(&mut u)?;
+            assert!(!ne.is_empty());
+            assert_eq!(
+                ne,
+                NonEmpty {
+                    head: 67305985,
+                    tail: vec![],
+                }
+            );
+            Ok(())
+        }
+
+        #[test]
+        fn test_arbitrary_with_tail() -> arbitrary::Result<()> {
+            let mut u = Unstructured::new(&[1, 2, 3, 4, 5, 6, 7, 8]);
+            let ne = NonEmpty::<i32>::arbitrary(&mut u)?;
+            assert!(!ne.is_empty());
+            assert_eq!(
+                ne,
+                NonEmpty {
+                    head: 67305985,
+                    tail: vec![526086],
+                }
+            );
+            Ok(())
+        }
+
+        #[test]
+        fn test_arbitrary_with_split() -> arbitrary::Result<()> {
+            let mut u = Unstructured::new(&[1, 2, 3, 4, 5, 6, 7, 8]);
+            let ne = NonEmpty::<i32>::arbitrary(&mut u)?;
+            let (head, middle, last) = ne.split();
+            let mut tail = middle.to_vec();
+            tail.extend(last);
+            assert_eq!(ne, NonEmpty { head: *head, tail });
+            Ok(())
+        }
+    }
 }
